// @HEADER
// ************************************************************************
//
//               Rapid Optimization Library (ROL) Package
//                 Copyright (2014) Sandia Corporation
//
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact lead developers:
//              Drew Kouri   (dpkouri@sandia.gov) and
//              Denis Ridzal (dridzal@sandia.gov)
//
// ************************************************************************
// @HEADER

#ifndef ROL_TRUSTREGIONFACTORY_H
#define ROL_TRUSTREGIONFACTORY_H

#include "ROL_Types.hpp"

#include "ROL_ParameterList.hpp"
#include "ROL_Ptr.hpp"

#include "ROL_TrustRegion.hpp"
#include "ROL_CauchyPoint.hpp"
#include "ROL_DogLeg.hpp"
#include "ROL_DoubleDogLeg.hpp"
#include "ROL_TruncatedCG.hpp"
#include "ROL_LinMore.hpp"

namespace ROL {
template<class Real>
  inline ROL::Ptr<TrustRegion<Real> > TrustRegionFactory(ROL::ParameterList &parlist) {
    ETrustRegion etr = StringToETrustRegion(
      parlist.sublist("Step").sublist("Trust Region").get("Subproblem Solver","Dogleg"));
    switch(etr) {
      case TRUSTREGION_CAUCHYPOINT:  return ROL::makePtr<CauchyPoint<Real>>(parlist);
      case TRUSTREGION_DOGLEG:       return ROL::makePtr<DogLeg<Real>>(parlist);
      case TRUSTREGION_DOUBLEDOGLEG: return ROL::makePtr<DoubleDogLeg<Real>>(parlist);
      case TRUSTREGION_TRUNCATEDCG:  return ROL::makePtr<TruncatedCG<Real>>(parlist);
      case TRUSTREGION_LINMORE:      return ROL::makePtr<LinMore<Real>>(parlist);
      default:                       return ROL::nullPtr;
    }
  }
}

#endif
